@@ -492,8 +492,8 @@ class getid3_real extends getid3_handler
 			$ParsedArray['fourcc'] = $ParsedArray['fourcc3'];
 
 		}
-		/** @var string[]|false[] $value */
-		foreach ($ParsedArray['comments'] as $key => $value) {
+		/** @var array<string|false> $value */
+		foreach ($ParsedArray['comments'] as $key => $value) { // @phpstan-ignore-line
 			if ($value[0] === false) {
 				$ParsedArray['comments'][$key][0] = '';
 			}
